jQuery(document).ready(function ($) {
	// Tooltips
	var tooltips = $('.five9-help-tip');
	five9_attach_tooltips( tooltips );

	$('input[type=checkbox][data-enables]').on( 'change', function(e) {
		var setting = $(this).data('enables');
		var target = $('.setting-' + setting);

		if ( $(this).is(':checked') ) {
			target.attr('readonly', false );
		} else {
			target.attr( 'readonly', true );
		}
	});

	$('select[data-enables]').on( 'change', function(e) {
		var setting = $(this).data('enables');
		var target = $('.setting-' + setting);

		if ( $(this).val() == $(this).data('enables-value' ) ) {
			target.attr('readonly', false );
		} else {
			target.attr( 'readonly', true );
		}
	});

	var selected_type = $('.setting-widget_type').val();
	var show_class    = selected_type + '-support';
	$('.five9.settings-group, .five9-repeatable-row, #five9-preview-on-page').each(function() {
		if ( ! $(this).hasClass( show_class ) ) {
			$(this).hide();
		} else {
			$(this).show();
		}
	});

	function validateEmail (e) {
		var invalid = function(email) {
			var regex = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
			return (!regex.test(email));
		};

		var value = $(this).val();
		if (value.length && invalid(value)) {
			$(this).css({borderColor: 'red'});
		}
		else {
			$(this).css({borderColor: ''});
		}
	}

	$('input[name="five9_form_fields[1][default_value]"]').on('keyup change', validateEmail);
	validateEmail.bind($('input[name="five9_form_fields[1][default_value]"]'))();

	$(document.body).on( 'change', '.setting-widget_type', function(e) {
		e.preventDefault();

		if ( 'proactive' === $(this).val() ) {
			$('#five9-preview-on-page').hide();
		} else {
			$('#five9-preview-on-page').show();
		}

		var show_class = $(this).val() + '-support';
		$('.five9.settings-group, .five9-repeatable-row, #five9-preview-on-page').each(function() {
			if ( ! $(this).hasClass( show_class ) ) {
				$(this).hide();
			} else {
				$(this).show();
			}
		});
	});

	$(document.body).on( 'click', '#five9-preview-on-page', function(e) {
		e.preventDefault();

		localStorage.clear();

		if (Five9SocialWidget.widgetAdded) {
			Five9SocialWidget.removeWidget();
		}

		var options = {};
		$('[data-api-param]').each( function(e) {
			if ( ! $(this).is(':visible')) {
				return;
			};

			if ( $(this).prop('readonly')) {
				return;
			}

			var setting = $(this).data('api-param');
			var group = false;
			if ( $(this).data('api-group') ) {
				var group = $(this).data('api-group');
			}

			if (setting === 'showProfile') {
				setting = 'showProfiles';
			}

			var type = $(this).prop('type');

			if ( group !== false ) {
				if ( typeof options.group === 'undefined' ) {
					options[ group ] = {};
				}

				var value = '';
				if ( 'text' === type ) {
					value = $(this).val();
				} else if ( 'checkbox' === type ) {
					value = $(this).is(':checked');
				} else if ( 'select-one' === type ) {
					value = $(this).find(':selected').val();
				}

				options[ group ][ setting ] = value;

			} else {

				var value = '';
				if ( 'text' === type ) {
					value = $(this).val();
				} else if ( 'checkbox' === type ) {
					value = $(this).is(':checked');
				} else if ( 'select-one' === type ) {
					value = $(this).find(':selected').val();
				}

				options[ setting ] = value;

			}

		});

		if ( 'custom' === options['theme'] ) {
			options['theme'] = $('.setting-custom_theme_css').val();
		}

		if ( $('.setting-enable_logo').is(':checked') ) {
			options['logo'] = $('.setting-logo_file_url').val();
		}

		if ( $('.setting-enable_custom_language').is(':checked')) {
			options['lang'] = $('.setting-enable_custom_language_url').val();
		}

		var form_fields = {};
		$('.five9-repeatable-row').each( function() {
			if ( ! $(this).is(':visible')) {
				return;
			}

			var type = $(this).find('input[data-form-input="field_type"]').val();
			if ( 'name' !== type && 'email' !== type && 'subject' !== type ) {
				type = $(this).find('input[data-form-input="field_name"]').val();
			}

			var show = $(this).find('input[data-form-input="show"]').is(':checked'),
				value = $(this).find('input[data-form-input="default_value"]').val(),
				label = $(this).find('input[data-form-input="display_label"]').val(),
				required = $(this).find('input[data-form-input="field_required"]').is(':checked');

			form_fields[ type ] = {
				value: value,
				show: show,
				label: label
			};

			if ( required ) {
				form_fields[ type ].required = required;
			}
		});

		options['fields'] = form_fields;
		var is_dev = $('input[name="is_dev"]').val();

		if ( '1' === is_dev ) {
			options['rootUrl'] = $('.five9-root-url-dev').val()+'/consoles/';
		} else {
			options['rootUrl'] = $('.setting-root_url').val()+'/consoles/';
		}

		if (options['profiles'].indexOf(',') !== -1) {
			options['profiles'] = options['profiles']
				.split(',')
				.map(
					function (profile) {
						return profile.trim();
					}
				);
		}

		Five9SocialWidget.addWidget(options);
		setTimeout(function () {
			$('#five9-maximize-button').click();
		}, 200);
	});

	var Five9_Settings = {
		init: function() {
			this.add();
			this.remove();
		},
		clone_repeatable: function(row, clear_data) {
			if ( typeof clear_data === 'undefined' ) {
				clear_data = true;
			}

			// Retrieve the highest current key
			var key = highest = 1;
			row.parent().find( '.five9-repeatable-row' ).each(function() {
				var current = $(this).data( 'key' );
				if( parseInt( current ) > highest ) {
					highest = current;
				}
			});
			key = highest += 1;

			clone = row.clone();

			/** manually update any select box values */
			clone.find( 'input' ).each(function() {
				$( this ).attr( 'disabled', false );
			});

			clone.removeClass( 'empty' );
			clone.attr( 'data-key', key );

			if ( clear_data ) {
				clone.find( 'td input[type="text"], select, textarea' ).val( '' );
			}

			clone.find( 'input, select, textarea' ).each(function() {
				var name = $( this ).attr( 'name' );

				name = name.replace( /\[(\d+)\]/, '[' + parseInt( key ) + ']');

				$( this ).attr( 'name', name ).attr( 'id', name );
			});

			clone.show();
			return clone;
		},
		add: function() {
			$( 'body' ).on( 'click', '.submit .five9-add-repeatable', function(e) {
				e.preventDefault();

				var row = $('.five9-repeatable-row.empty'),
					clone = Five9_Settings.clone_repeatable(row);
				clone.insertBefore( row );
			});
		},
		remove: function() {
			$( 'body' ).on( 'click', '.five9-remove-repeatable', function(e) {
				e.preventDefault();

				var row = $(this).parent().parent( 'tr' );
				row.fadeOut( 'fast' ).remove();

				/* re-index after deleting */
				$('tr.five9-repeatable-row').not('.empty').each( function( rowIndex ) {
					$(this).find( 'input, select' ).each(function() {
						var name = $( this ).attr( 'name' );
						name = name.replace( /\[(\d+)\]/, '[' + rowIndex+ ']');
						$( this ).attr( 'name', name ).attr( 'id', name );
					});
				});
			});
		},
	}
	Five9_Settings.init();

})

function five9_attach_tooltips( selector ) {
	// Tooltips
	selector.tooltip({
		content: function() {
			return jQuery(this).prop('title');
		},
		tooltipClass: 'five9-ui-tooltip',
		position: {
			my: 'center top',
			at: 'center bottom+10',
			collision: 'flipfit'
		},
		hide: {
			duration: 200
		},
		show: {
			duration: 200
		}
	});
}
