<?php
/**
 * Plugin Name: Five9 - Live Chat for WordPress
 * Description: Enable Five9's Live Chat on your WordPress site.
 * Author: Five9, Inc.
 * Author URI: https://www.five9.com
 * Version: 1.1.2
 * Text Domain: five9
 * Domain Path: languages
 *
 * Five9 - Live Chat for WordPress is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Five9 - Live Chat for WordPress is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Five9 - Live Chat for WordPress. If not, see <http://www.gnu.org/licenses/>.
 *
 * @category Core
 * @author Five9, Inc.
 * @version 1.1.0
 */

final class Five9 {

	/** Singleton *************************************************************/

	/**
	 * @var Five9 The one true \Five9
	 * @since 1.0
	 */
	private static $instance;

	/**
	 * Main Five9 Instance.
	 *
	 * Insures that only one instance of Five9 exists in memory at any one
	 * time. Also prevents needing to define globals all over the place.
	 *
	 * @since 1.0
	 * @static
	 * @staticvar array $instance
	 * @uses Five9::setup_constants() Setup the constants needed.
	 * @uses Five9::includes() Include the required files.
	 * @uses Five9::load_textdomain() load the language files.
	 * @see EDD()
	 * @return object|Five9 The one true Five9
	 */
	public static function instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Five9 ) ) {
			self::$instance = new Five9;
			self::$instance->setup_constants();
			self::$instance->includes();
		}

		return self::$instance;
	}

	/**
	 * Throw error on object clone.
	 *
	 * The whole idea of the singleton design pattern is that there is a single
	 * object therefore, we don't want the object to be cloned.
	 *
	 * @since 1.0
	 * @access protected
	 * @return void
	 */
	public function __clone() {
		// Cloning instances of the class is forbidden.
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'five9' ), '1.0' );
	}

	/**
	 * Disable unserializing of the class.
	 *
	 * @since 1.0
	 * @access protected
	 * @return void
	 */
	public function __wakeup() {
		// Unserializing instances of the class is forbidden.
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'five9' ), '1.0' );
	}

	/**
	 * Setup plugin constants.
	 *
	 * @access private
	 * @since 1.0
	 * @return void
	 */
	private function setup_constants() {

		// Plugin version.
		if ( ! defined( 'FIVE9_VERSION' ) ) {
			define( 'FIVE9_VERSION', '1.0.5' );
		}

		// Plugin Folder Path.S
		if ( ! defined( 'FIVE9_PLUGIN_DIR' ) ) {
			define( 'FIVE9_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
		}

		// Plugin Folder URL.
		if ( ! defined( 'FIVE9_PLUGIN_URL' ) ) {
			define( 'FIVE9_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
		}

		// Plugin Root File.
		if ( ! defined( 'FIVE9_PLUGIN_FILE' ) ) {
			define( 'FIVE9_PLUGIN_FILE', __FILE__ );
		}
	}

	/**
	 * Include required files.
	 *
	 * @access private
	 * @since 1.4
	 * @return void
	 */
	private function includes() {
		// Global Files.
		require_once FIVE9_PLUGIN_DIR . 'includes/class-settings.php';
		require_once FIVE9_PLUGIN_DIR . 'includes/class-forms.php';
		require_once FIVE9_PLUGIN_DIR . 'includes/class-widget.php';

		if ( is_admin() ) {
			require_once FIVE9_PLUGIN_DIR . 'includes/admin/class-admin.php';
			require_once FIVE9_PLUGIN_DIR . 'includes/admin/class-html-elements.php';

		} else {
			// Frontend Only Files
		}
	}

}

function five9() {
	return Five9::instance();
}
add_action( 'plugins_loaded', 'five9' );
