<?php

namespace Five9\Admin;

use Five9\Settings;
use Five9\Forms;
use Five9\HTML;

class Pages {
	private static $instance;
	private static $settings;

	public static function instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Pages ) ) {
			self::$instance = new Pages;
			self::$settings = Settings::instance();

			self::$instance->actions();
			self::$instance->filters();
		}

		return self::$instance;
	}

	private function actions() {
		add_action( 'admin_init', array( $this, 'register_settings_section' ) );
		add_action( 'admin_menu', array( $this, 'add_settings_page' ) );

		add_action( 'Five9\Admin\Pages\render_form_fields', array( $this, 'render_form_fields' ) );
	}

	private function filters() {}

	public function register_settings_section() {
		add_settings_section('five9_main', __( 'Configure Chat', 'five9' ), array( $this, 'output_settings' ), 'five9');
	}

	public function add_settings_page() {
		add_options_page(
			__( 'Five9 Chat', 'five9' ),
			__( 'Five9 Chat', 'five9' ),
			'manage_options',
			'five9',
			array( $this, 'settings_page_callback' )
		);
	}

	public function settings_page_callback() {
		?>
		<div class="wrap">
			<h2><?php _e( 'Five9 Chat', 'five9' ); ?></h2>

			<form id="five9-chat-settings" class="five9-chat-settings" action="options.php" method="post">
				<?php do_settings_sections('five9'); ?>
				<?php settings_fields( 'five9_settings' ); ?>

				<div class="final-wrap">
					<input type="hidden" class="setting-root_url" name="five9_settings[root_url]" value="https://app.five9.com" />
					<?php if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) :?>
					<p>
						<label><?php _e( 'Root URL', 'five9' ); ?></label>
						<input class="five9-root-url-dev" type="text" name="five9_settings[root_url_dev]" value="<?php echo esc_attr( self::$settings->get_setting( 'root_url_dev' ) ); ?>" />
						<input type="hidden" name="is_dev" value="1" />
					</p>
					<?php else: ?>
						<input type="hidden" name="is_dev" value="0" />
					<?php endif; ?>
					<input type="Submit" value="<?php _e( 'Save Settings', 'five9' ); ?>" class="button-primary" />
					&nbsp;
					<a href="#" id="five9-preview-on-page" class="button-secondary chat-support email-support"><?php _e( 'Preview On Page', 'five9' ); ?></a>
				</div>
			</form>
		</div><!-- .wrap -->
		<?php
	}

	public function output_settings() {
		$settings_columns  = self::$settings->get_settings_columns();
		$settings_groups   = self::$settings->get_settings_groups();
		?>
		<div class="metabox-holder">
			<?php foreach ( $settings_columns as $col => $col_groups ) : ?>
			<div id="five9-postbox-container-<?php echo $col; ?>" class="postbox-container">
				<?php
				foreach ( $col_groups['groups'] as $group_id ) {
					$this->render_group( $group_id, $settings_groups[ $group_id ] );
				}
				?>
			</div>
			<?php endforeach; ?>
		</div>

		<div class="metabox-holder five9-form-fields">
			<div id="five9-postbox-container-form-fields" class="postbox-container full">
				<div class="five9 postbox">
				<?php do_action( 'Five9\Admin\Pages\render_form_fields' ); ?>
				</div>
			</div>
		</div>
		<?php
	}

	private function render_group( $group_id, $group_data ) {
		$support_classes = '';
		if ( ! empty( $group_data['support'] ) ) {
			foreach( $group_data['support'] as $support ) {
				$support_classes .= ' ' . sanitize_html_class( $support ) . '-support ';
			}
		}
		?>
		<div id="<?php echo esc_attr( $group_id ); ?>" class="five9 postbox settings-group <?php echo $support_classes; ?>">
			<h2 class="hndle">
				<?php echo $group_data['label']; ?>
				<?php if ( ! empty( $group_data['tooltip'] ) ) : ?>
					<span alt="f223" class="five9-help-tip dashicons dashicons-editor-help" title="<?php echo esc_attr( $group_data['tooltip'] ); ?>"></span>
				<?php endif; ?>
			</h2>
			<div class="inside">
				<div class="settings">
					<?php foreach ( $group_data['settings'] as $setting_id ) : ?>
						<?php $this->output_setting_field( $setting_id ); ?>
					<?php endforeach; ?>
				</div>
			</div>
		</div>
		<?php
	}

	private function output_setting_field( $setting_field = '' ) {
		if ( empty( $setting_field ) ) {
			return;
		}

		$setting = self::$settings->get_setting_definition( $setting_field );
		HTML::render_setting( $setting_field, $setting );

	}

	public function render_form_fields() {
		$form_fields = Forms::get_form_field_settings();
		$field_types = Forms::form_field_types();
		?>

		<div id="five9-field-wrapper" class="five9-postbox">
			<h2 class="hndle">
				<?php _e( 'Form Fields', 'five9' ); ?>
			</h2>
			<div id="five9-fields" class="inside">
				<div>
					<table class="widefat five9-repetable-table" width="100%" cellpadding="0" cellspacing="0">
						<thead>
						<tr>
							<th style="width: 25%"><?php _e( 'Field Name', 'five9' ); ?></th>
							<th style="width: 25%"><?php _e( 'Display Label', 'five9' ); ?></th>
							<th style="width: 25%"><?php _e( 'Default Value', 'five9' ); ?></th>
							<th class="center" style="width: 15%"><?php _e( 'Show?', 'five9' ); ?></th>
							<th class="center" style="width: 15%"><?php _e( 'Required?', 'five9' ); ?></th>
							<th class="center" style="width: 10%"><?php _e( 'Actions', 'five9' ); ?></th>
						</tr>
						</thead>
						<tbody>

							<?php foreach ( $form_fields as $key => $field ) : ?>
								<?php
								$supports = $field_types[ $field['field_type'] ]['supports'];
								$support_classes = '';
								if ( ! empty( $supports ) ) {
									foreach( $supports as $support ) {
										$support_classes .= ' ' . sanitize_html_class( $support ) . '-support ';
									}
								}
								$required_field = false;
								?>
								<tr class="five9-repeatable-row <?php echo $support_classes; ?>" data-key="<?php echo esc_attr( $key ); ?>">
									<input type="hidden" data-form-input="field_type" name="five9_form_fields[<?php echo $key; ?>][field_type]" value="<?php echo esc_attr( $field['field_type'] ); ?>" />

									<td>
										<?php if ( in_array( 'field_name', $field_types[ $field['field_type'] ]['fields'] ) ) : ?>
											<input type="text" data-form-input="field_name" name="five9_form_fields[<?php echo $key; ?>][field_name]" placeholder="<?php echo esc_attr( 'Field Name', 'five9' ); ?>" value="<?php echo esc_attr( $field['field_name'] ); ?>" />
										<?php else: ?>
											<?php echo esc_html( $field['field_name'] ); ?>
											<input type="hidden" data-form-input="field_name" name="five9_form_fields[<?php echo $key; ?>][field_name]" value="<?php echo esc_attr( $field['field_name'] ); ?>" />
										<?php endif; ?>
									</td>

									<td>
										<?php if ( in_array( 'display_label', $field_types[ $field['field_type'] ]['fields'] ) ) : ?>
											<input type="text" data-form-input="display_label" name="five9_form_fields[<?php echo $key; ?>][display_label]" placeholder="<?php echo esc_attr( 'Display Label', 'five9' ); ?>" value="<?php echo esc_attr( $field['display_label'] ); ?>" />
										<?php else: ?>
											<?php echo esc_html( $field['display_label'] ); ?>
											<input type="hidden" data-form-input="display_label" name="five9_form_fields[<?php echo $key; ?>][display_label]" value="<?php echo esc_attr( $field['display_label'] ); ?>" />
										<?php endif; ?>
									</td>

									<td>
										<?php if ( in_array( 'default_value', $field_types[ $field['field_type'] ]['fields'] ) ) : ?>
											<input type="text" data-form-input="default_value" name="five9_form_fields[<?php echo $key; ?>][default_value]" placeholder="<?php echo esc_attr( 'Default Value', 'five9' ); ?>" value="<?php echo esc_attr( $field['default_value'] ); ?>" />
										<?php else: ?>
											<?php if ( ! empty( $field_types[ $field['field_type'] ]['description'] ) ) : ?>
												<span alt="f223" class="five9-help-tip dashicons dashicons-editor-help" title="<?php echo esc_attr( $field_types[ $field['field_type'] ]['description'] ); ?>"></span>
											<?php endif; ?>
											<input type="hidden" data-form-input="default_value" name="five9_form_fields[<?php echo $key; ?>][default_value]" value="" />
										<?php endif; ?>
									</td>

									<td class="center" >
										<input type="hidden" name="five9_form_fields[<?php echo $key; ?>][show]" value="0" />
										<?php if ( ! filter_var( $field_types[ $field['field_type'] ]['must_show'], FILTER_VALIDATE_BOOLEAN ) ) : ?>
											<input type="checkbox" data-form-input="show" name="five9_form_fields[<?php echo $key; ?>][show]" value="1" <?php checked( 1, absint( $field['show'] ) ); ?> />
										<?php elseif ( isset( $field['hidden'] ) && filter_var( $field['hidden'], FILTER_VALIDATE_BOOLEAN ) ) : ?>
											<?php _e( 'Hidden field', 'five9' ); ?>
											<span alt="f223" class="five9-help-tip dashicons dashicons-editor-help" title="<?php echo esc_attr( __( 'This is a hidden field, and will only show to chat agents.', 'five9' ) ); ?>"></span>
										<?php else: ?>
											<?php $required_field = true; ?>
											<input type="checkbox" checked="checked" onClick="return false;" readonly="readonly" data-form-input="show" name="five9_form_fields[<?php echo $key; ?>][show]" value="1" />
											<input type="hidden" name="five9_form_fields[<?php echo $key; ?>][must_show]" value="1" />
										<?php endif; ?>
									</td>

									<td class="center" >
										<input type="hidden" name="five9_form_fields[<?php echo $key; ?>][field_required]" value="0" />
										<?php if ( $field_types[ $field['field_type'] ]['required'] ): ?>
											<input type="checkbox" onClick="return false;" readonly="readonly" checked="checked" value="1" />
										<?php else: ?>
											<?php $is_checked = (isset($field['field_required']) && $field['field_required']); ?>
											<input type="checkbox" data-form-input="field_required" name="five9_form_fields[<?php echo $key; ?>][field_required]" value="1" <?php if ($is_checked):?>checked="checked"<?php endif;?> />
											<?php if ($is_checked) {$required_field = true;} ?>
										<?php endif;?>
									</td>

									<td class="five9-action-icons center">
										<?php if ( $required_field ) : ?>
											<span alt="f223" class="five9-help-tip dashicons dashicons-editor-help" title="<?php echo esc_attr( __( 'This is a required field, and will always be shown to the user.', 'five9' ) ); ?>"></span>
										<?php endif; ?>
										<?php if ( ! $field_types[ $field['field_type'] ]['required'] ) : ?>
											<a title="<?php _e( 'Delete', 'five9' ); ?>" href="#" class="five9-remove-repeatable"><span class="dashicons dashicons-dismiss"></span></a>
										<?php endif; ?>
									</td>

								</tr>
							<?php endforeach; ?>

							<tr class="five9-repeatable-row empty" data-key="0">
								<input disabled="disabled" data-form-input="field_type" type="hidden" name="five9_form_fields[0][field_type]" value="text" />

								<td>
									<input disabled="disabled" data-form-input="field_name" type="text" name="five9_form_fields[0][field_name]" placeholder="<?php echo esc_attr( 'Field Name', 'five9' ); ?>" value="" />
								</td>

								<td>
									<input disabled="disabled" data-form-input="display_label" type="text" name="five9_form_fields[0][display_label]" placeholder="<?php echo esc_attr( 'Display Label', 'five9' ); ?>" value="" />
								</td>

								<td>
									<input disabled="disabled" data-form-input="default_value" type="text" name="five9_form_fields[0][default_value]" placeholder="<?php echo esc_attr( 'Default Value', 'five9' ); ?>" value="" />
								</td>

								<td class="center" >
									<input disabled="disabled" data-form-input="show" type="checkbox" name="five9_form_fields[0][show]" value="1" />
								</td>

								<td class="center" >
									<input disabled="disabled" data-form-input="field_required" type="checkbox" name="five9_form_fields[0][field_required]" value="1" />
								</td>

								<td class="five9-action-icons center">
									<a title="<?php _e( 'Delete', 'five9' ); ?>" href="#" class="five9-remove-repeatable"><span class="dashicons dashicons-dismiss"></span></a>
								</td>
							</tr>

							<tr class="five9-add-repeatable-wrapper">
								<td class="submit" colspan="4" style="float: none; clear:both; background:#fff;">
									<a class="button-secondary five9-add-repeatable" style="margin: 6px 0;"><?php _e( 'Add New Field', 'five9' ); ?></a>
								</td>
							</tr>
						</tbody>
					</table>
				</div>
			</div>
			<style type="text/css">
			.five9-frame .five9-frame-full .five9-header {
				overflow: hidden;
			}
			</style>
		</div>
		<?php
	}
}
Pages::instance();
