<?php

namespace Five9;

class HTML {

	public static function render_setting( $setting_id, $setting ) {
		$setting_type_method = $setting['type'];
		if ( is_callable( array( __CLASS__, $setting_type_method ) ) ) {
			$current_value = Settings::get_setting( $setting_id );
			?><p class="five9-setting-wrapper"><?php
				\Five9\HTML::$setting_type_method( $setting, $setting_id, $current_value );
			?></p><?php
			if ( ! empty( $setting['description'] ) ) {
				?><p><?php echo $setting['description']; ?></p><?php
			}
		}
	}

	private static function select( $setting, $setting_id, $current_value ) {
		if ( false === $current_value || ! array_key_exists( $current_value, $setting['options'] ) ) {
			$selected = $setting['default'];
		} elseif ( ! empty( $current_value ) ) {
			$selected = $current_value;
		}
		?>
		<label for="five9_settings[<?php echo $setting_id; ?>]">
			<?php echo esc_html( $setting['label'] ) ?>
			<?php if ( ! empty( $setting['tooltip'] ) ) : ?>
				<span alt="f223" class="five9-help-tip dashicons dashicons-editor-help" title="<?php echo esc_attr( $setting['tooltip'] ); ?>"></span>
			<?php endif; ?>
		</label>
		<select
			id="five9_settings[<?php echo $setting_id; ?>]"
			name="five9_settings[<?php echo $setting_id; ?>]"
			class="setting-<?php echo sanitize_html_class( $setting_id ); ?>"
			<?php
			if ( ! empty( $setting['enables'] ) ) {
				echo ' data-enables="' . esc_attr( $setting['enables']['setting'] ) . '" ';
				echo ' data-enables-value="' . esc_attr( $setting['enables']['value'] ) . '" ';
			}
			if ( ! empty( $setting['api_param'] ) ) {
				if ( ! empty( $setting['api_param']['name'] ) ) {
					echo ' data-api-param="' . esc_attr( $setting['api_param']['name'] ) . '" ';
				}

				if ( ! empty( $setting['api_param']['group'] ) ) {
					echo ' data-api-group="' . esc_attr( $setting['api_param']['group'] ) . '" ';
				}
			}
			?>
		>
			<?php foreach ( $setting['options'] as $value => $label ) : ?>
				<option
					value="<?php echo esc_attr( $value ); ?>"
					<?php echo ( $value === $selected ) ? 'selected="selected"' : ''; ?>
				>
					<?php echo esc_html( $label ); ?>
				</option>
			<?php endforeach; ?>
		</select>
		<?php
	}

	private static function text( $setting, $setting_id, $current_value ) {
		?>
		<label for="five9_settings[<?php echo $setting_id; ?>]">
			<?php echo esc_html( $setting['label'] ) ?>
			<?php if ( ! empty( $setting['tooltip'] ) ) : ?>
				<span alt="f223" class="five9-help-tip dashicons dashicons-editor-help" title="<?php echo esc_attr( $setting['tooltip'] ); ?>"></span>
			<?php endif; ?>
		</label>
		<?php
		$readonly = self::is_readonly( $setting );
		$value = ! empty( $current_value ) ? $current_value : '';
		if ( empty( $value ) && ! empty( $setting['default'] ) ) {
			$value = $setting['default'];
		}
		?>
		<input type="text"
			   id="five9_settings[<?php echo $setting_id; ?>]"
			   name="five9_settings[<?php echo $setting_id; ?>]"
			   value="<?php echo esc_attr( $value ); ?>"
			   placeholder="<?php echo esc_attr( $setting['placeholder'] ); ?>"
			   class="setting-<?php echo sanitize_html_class( $setting_id ); ?>"
			   <?php echo $readonly; ?>
				<?php
				if ( ! empty( $setting['enables'] ) ) {
					echo ' data-enables="' . esc_attr( $setting['enables']['setting'] ) . '" ';
					echo ' data-enables-value="' . esc_attr( $setting['enables']['value'] ) . '" ';
				}

				if ( ! empty( $setting['api_param'] ) ) {
					if ( ! empty( $setting['api_param']['name'] ) ) {
						echo ' data-api-param="' . esc_attr( $setting['api_param']['name'] ) . '" ';
					}

					if ( ! empty( $setting['api_param']['group'] ) ) {
						echo ' data-api-group="' . esc_attr( $setting['api_param']['group'] ) . '" ';
					}
				}
				?>
		/>
		<?php
	}

	private static function checkbox( $setting, $setting_id, $current_value ) {
		$checked = '';
		if (
			( false === $current_value && true === $setting['default'] ) ||
			! empty( $current_value )
		) {
			$checked = 'checked="checked"';
		}
		?>
		<input type="hidden" name="five9_settings[<?php echo $setting_id; ?>]" value="0" />
		<label for="five9_settings[<?php echo $setting_id; ?>]">
			<?php echo esc_html( $setting['label'] ) ?>
			<?php if ( ! empty( $setting['tooltip'] ) ) : ?>
				<span alt="f223" class="five9-help-tip dashicons dashicons-editor-help" title="<?php echo esc_attr( $setting['tooltip'] ); ?>"></span>
			<?php endif; ?>
			&nbsp;
			<input type="checkbox"
				   id="five9_settings[<?php echo $setting_id; ?>]"
				   name="five9_settings[<?php echo $setting_id; ?>]"
				   <?php echo $checked; ?>
				   value="1"
				   class="setting-<?php echo sanitize_html_class( $setting_id ); ?>"
					<?php
					if ( ! empty( $setting['enables'] ) ) {
						echo ' data-enables="' . esc_attr( $setting['enables']['setting'] ) . '" ';
						echo ' data-enables-value="' . esc_attr( $setting['enables']['value'] ) . '" ';
					}

					if ( ! empty( $setting['api_param'] ) ) {
						if ( ! empty( $setting['api_param']['name'] ) ) {
							echo ' data-api-param="' . esc_attr( $setting['api_param']['name'] ) . '" ';
						}

						if ( ! empty( $setting['api_param']['group'] ) ) {
							echo ' data-api-group="' . esc_attr( $setting['api_param']['group'] ) . '" ';
						}
					}
					?>
			/>
		</label>
		<?php
	}

	private static function is_readonly( $setting ) {
		$readonly = '';
		if ( ! empty( $setting['requires'] ) ) {
			$requirements_met = true;
			foreach ( $setting['requires'] as $setting => $value ) {
				$required_setting = Settings::get_setting( $setting );
				if ( $required_setting != $value ) {
					$requirements_met = false;
					break;
				}
			}

			if ( false === $requirements_met ) {
				$readonly = 'readonly="readonly"';
			}
		}

		return $readonly;
	}

}
