<?php

namespace Five9;

class Forms {
	private static $instance;

	public static function instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Forms ) ) {
			self::$instance = new Forms;

			self::$instance->actions();
		}

		return self::$instance;
	}

	private function actions() {
		add_action( 'admin_init', array( $this, 'register_setting' ) );
	}

	public function register_setting() {
		register_setting( 'five9_settings', 'five9_form_fields', array( $this, 'sanitize_form_fields' ) );
	}

	public function sanitize_form_fields( $input ) {

		foreach( $input as $key => $row ) {
			$row = array_filter( $row );

			// Empty any rows that have no values
			if ( 1 === count( $row ) && array_key_exists( 'field_type', $row ) ) {
				unset( $input[ $key ] );
				continue;
			}

			$input[ $key ]['field_type'] = sanitize_text_field( $row['field_type'] );
			$input[ $key ]['field_name'] = sanitize_text_field( $row['field_name'] );
			$input[ $key ]['display_label'] = sanitize_text_field( $row['display_label'] );
			$input[ $key ]['default_value'] = ! empty( $row['default_value'] ) ? sanitize_text_field( $row['default_value'] ) : '';
			$input[ $key ]['show'] = ! empty( $row['show'] ) ? "1" : "0";
			$input[ $key ]['field_required'] = ! empty( $row['field_required'] ) ? "1" : "0";
			$input[ $key ]['must_show'] = ! empty( $row['must_show'] ) ? "1" : "0";
		}

		return array_values( $input );

	}

	public static function get_form_field_settings() {
		$field_types = self::form_field_types();
		$default_form_fields = array(
			array(
				'field_type'    => 'name',
				'field_name'    => $field_types['name']['field_name'],
				'display_label' => $field_types['name']['field_name'],
				'default_value' => '',
				'show'          => $field_types['name']['show'],
			),
			array(
				'field_type'    => 'email',
				'field_name'    => $field_types['email']['field_name'],
				'display_label' => $field_types['email']['field_name'],
				'default_value' => '',
				'show'          => $field_types['email']['show'],
			),
			array(
				'field_type'    => 'subject',
				'field_name'    => $field_types['subject']['field_name'],
				'display_label' => $field_types['subject']['field_name'],
				'default_value' => '',
				'show'          => $field_types['subject']['show'],
			)
		);

		$form_fields = get_option( 'five9_form_fields', $default_form_fields );
		if ( ! is_array( $form_fields ) ) {
			$form_fields = $default_form_fields;
		}

		return apply_filters( 'Five9\Forms\get_form_field_settings', $form_fields );
	}

	public static function form_field_types() {
		$form_field_types = array(
			'name' => array(
				'field_name' => __( 'Name', 'five9' ),
				'required'   => true,
				'show'       => true,
				'hidden'     => false,
				'must_show'  => true,
				'fields'     => array(
					'display_label', 'default_value'
				),
				'supports' => array(
					'chat', 'email', 'proactive',
				)
			),
			'email' => array(
				'field_name' => __( 'Email', 'five9' ),
				'required'   => true,
				'show'       => true,
				'hidden'     => false,
				'must_show'  => false,
				'fields'     => array(
					'display_label', 'default_value'
				),
				'supports' => array(
					'chat', 'email', 'proactive',
				)
			),
			'subject' => array(
				'field_name' => __( 'Subject', 'five9' ),
				'required'   => true,
				'show'       => true,
				'hidden'     => false,
				'must_show'  => true,
				'fields'     => array(
					'display_label', 'default_value'
				),
				'supports' => array(
					'email',
				)
			),
			'text' => array(
				'field_name' => '',
				'required'   => false,
				'show'       => true,
				'hidden'     => false,
				'must_show'  => false,
				'field_required' => false,
				'fields'     => array(
					'field_name', 'display_label', 'default_value'
				),
				'supports' => array(
					'chat', 'email', 'proactive',
				)
			),
		);

		return apply_filters( 'Five9\Forms\form_field_types', $form_field_types );
	}

}

Forms::instance();
