<?php

namespace Five9;

use Settings;

class Widget {

	private static $instance;

	public static function instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Widget ) ) {
			self::$instance = new Widget;
		}

		self::$instance->actions();

		return self::$instance;
	}

	private function actions() {
		add_action( 'admin_enqueue_scripts', array( $this, 'load_widget_script' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'load_widget_script' ) );
		add_action( 'init', array( $this, 'register_shortcode' ) );
	}

	public function should_load_widget() {
		$return = true;

		if ( is_admin() ) {
			$current_screen = get_current_screen();
			if ( 'settings_page_five9' !== $current_screen->base ) {
				$return = false;
			}
		}

		return apply_filters( 'Five9\Widget\should_load_widget', $return );
	}

	public function load_widget_script() {
		$widget_type   = \Five9\Settings::get_setting( 'widget_type' );
		$root_url      = $this->get_root_url();
		$widget_source = $root_url . 'consoles/SocialWidget/five9-social-widget.min.js';
		wp_register_script( 'five9-widget', $widget_source, array( 'jquery' ), null, false );

		$proactive_styles = $root_url . 'consoles/ProactiveChat/stylesheets/five9proactivechat.css';
		wp_register_style( 'five9-proactive', $proactive_styles, array(), null );

		$proactive_source = $root_url . 'consoles/ProactiveChat/javascripts/five9proactivechat.js';
		wp_register_script( 'five9-proactive', $proactive_source, array( 'jquery' ), null, false );

		$js_dir  = FIVE9_PLUGIN_URL . 'assets/js/';

		// Use minified libraries if SCRIPT_DEBUG is turned off
		$suffix  = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';
		wp_register_script( 'five9-frontend', $js_dir . 'frontend' . $suffix . '.js' , array( 'jquery' ), FIVE9_VERSION, false );




		if ( ! is_admin() ) {
			$enabled = \Five9\Settings::get_setting( 'enabled' );
			if ( empty( $enabled ) ) {
				wp_enqueue_script( 'five9-frontend' );
				return;
			}

			if ( 'proactive' === $widget_type ) {
				wp_enqueue_style( 'five9-proactive' );

				wp_enqueue_script( 'five9-proactive' );
				wp_add_inline_script( 'five9-proactive', '/* < ![CDATA[ */ const $ = jQuery; /* ]]> */', 'before' );

				wp_localize_script( 'five9-frontend', 'five9_widget_options', self::get_widget_config( $widget_type ) );
			} else {
				// Load the Social Widget
				wp_localize_script( 'five9-frontend', 'five9_widget_options', self::get_widget_config( $widget_type ) );
				wp_enqueue_script( 'five9-widget' );
			}

			wp_enqueue_script( 'five9-frontend' );

		} else {
			wp_enqueue_script( 'five9-widget' );
		}

	}

	public function register_shortcode() {
		add_shortcode( 'five9-chat', array( $this, 'render_shortcode' ) );
	}

	public function render_shortcode( $atts ) {
		$a = shortcode_atts( array(
			'text' => __( 'Chat Now', 'five9' ),
			'width' => __('320', 'five9'),
			'height' => __('550', 'five9'),
			'location' => __('no', 'five9'),
			'menubar' => __('no', 'five9'),
			'resizable' => __('yes', 'five9'),
			'scrollbars' => __('no', 'five9'),
			'status' => __('no', 'five9'),
			'titlebar' => __('no', 'five9'),
			'toolbar' => __('no', 'five9')
		), $atts );

		$type = \Five9\Settings::get_setting( 'widget_type' );

		if ( 'proactive' === $type ) {
			return '';
		}

		$chat_url  = self::get_root_url() . 'consoles/ChatConsole/index.html?';
		$chat_args = self::get_widget_config( 'chat' );

		unset( $chat_args['options']['rootUrl'] );

		$first     = true;
		foreach( $chat_args['options'] as $key => $value ) {

			if ( false === $first ) {
				$chat_url .= '&';
			}

			$chat_url .= $key . '=';

			if ($key == 'profiles') {
				if (is_array($value)) {
					$value = implode( ',', $value );
				}
			}

			if ( is_array( $value ) ) {
				$value = json_encode( $value );
			}

			$chat_url .= rawurlencode(
				is_bool( $value ) ? ($value ? 'true' : 'false') : $value
			);

			$first = false;
		}

		$chat_link = '<a class="five9-chat-wp-link" target="_blank" href="' . $chat_url .'" ';
		$chat_link .= "data-options='{\"width\":{$a['width']},";
		$chat_link .=	"\"height\":{$a['height']},";
		$chat_link .= "\"location\":\"{$a['location']}\",";
		$chat_link .= "\"menubar\":\"{$a['menubar']}\",";
		$chat_link .= "\"resizable\":\"{$a['resizable']}\",";
		$chat_link .= "\"scrollbars\":\"{$a['scrollbars']}\",";
		$chat_link .= "\"status\":\"{$a['status']}\",";
		$chat_link .= "\"titlebar\":\"{$a['titlebar']}\",";
		$chat_link .= "\"toolbar\":\"{$a['toolbar']}\"}' ";
		$chat_link .= '>' . $a['text'] . '</a>';

		return $chat_link;
	}

	private static function get_root_url() {
		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			$root_url = \Five9\Settings::get_setting( 'root_url_dev' );
		}

		$root_url = empty( $root_url ) ? \Five9\Settings::get_setting( 'root_url' ) : $root_url;

		return trailingslashit( $root_url );
	}

	public static function get_widget_config( $type = 'chat', $format = 'array', $settings = array() ) {
		$config = array();
		if ( empty( $settings ) ) {
			$all_settings = \Five9\Settings::get_settings();
		}
		$config['type'] = $type;
		switch( $type ) {
			case 'chat':
				$config['options'] = array(
					'rootUrl' => self::get_root_url() . 'consoles/',
					'type' => 'chat',
					'title' => $all_settings['title'],
					'tenant' => $all_settings['domain'],
					'profiles' => self::parse_profiles($all_settings['profiles']),
					'showProfiles' => filter_var( $all_settings['show_profile'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'profileLabel' => $all_settings['profile_label'],
					'autostart' => filter_var( $all_settings['auto_start'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'surveyOptions' => array(
						'showComment'    => filter_var( $all_settings['show_comments_field'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
						'requireComment' => filter_var( $all_settings['require_comments_field'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					),
					'playSoundOnMessage' => filter_var( $all_settings['play_sounds'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'allowCustomerToControlSoundPlay' => filter_var( $all_settings['allow_disable_sounds'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'showEmailButton' => filter_var( $all_settings['show_email_button'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'useBusinessHours' => filter_var( $all_settings['enable_business_hours'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'hideDuringAfterHours' => filter_var( $all_settings['hide_widget_after_hours'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'showPrintButton' => filter_var( $all_settings['show_print_button'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'allowUsabilityMenu' => filter_var( $all_settings['show_usability_menu'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'enableCallback' => filter_var( $all_settings['enable_callback'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
				);

				if ( ! empty( $all_settings['callback_list'] ) ) {
					$config['options']['callbackList'] = $all_settings['callback_list'];
				}

				if ( ! empty( $all_settings['namespace'] ) ) {
					$config['options']['namespace'] = $all_settings['namespace'];
				}

				if ( ! empty( $all_settings['ga_id'] ) ) {
					$config['options']['ga'] = $all_settings['ga_id'];
				}

				// Possibly add the logo
				if ( filter_var( $all_settings['enable_logo'], FILTER_VALIDATE_BOOLEAN )) {
					$config['options']['logo'] = $all_settings['logo_file_url'];
				}

				// Custom Language
				if ( filter_var( $all_settings['enable_custom_language'], FILTER_VALIDATE_BOOLEAN ) ) {
					$config['options']['lang'] = $all_settings['custom_language_url'];
				}

				// Conditionally load the themes.
				if ( 'custom' === $all_settings['theme'] ) {
					$config['options']['theme'] = $all_settings['custom_theme_css'];
				} else {
					$config['options']['theme'] = $all_settings['theme'];
				}

				$config['options']['fields'] = self::get_form_fields_config( $type );
				break;

			case 'email':
				$config['options'] = array(
					'rootUrl' => self::get_root_url() . 'consoles/',
					'type' => 'email',
					'title' => $all_settings['title'],
					'tenant' => $all_settings['domain'],
					'profiles' => self::parse_profiles($all_settings['profiles']),
					'showProfiles' => filter_var( $all_settings['show_profile'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
				);

				// Possibly add the logo
				if ( filter_var( $all_settings['enable_logo'], FILTER_VALIDATE_BOOLEAN ) ) {
					$config['options']['logo'] = $all_settings['logo_file_url'];
				}

				// Conditionally load the themes.
				if ( 'custom' === $all_settings['theme'] ) {
					$config['options']['theme'] = $all_settings['custom_theme_css'];
				} else {
					$config['options']['theme'] = $all_settings['theme'];
				}

				$config['options']['fields'] = self::get_form_fields_config( $type );

				break;

			case 'proactive':
				$config['options'] = array(
					'title'            => $all_settings['title'],
					'tenant'           => $all_settings['domain'],
					'showProfiles'     => filter_var( $all_settings['show_profile'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
					'restAPI'          => $all_settings['rest_api_url'],
					'chatConsole'      => $all_settings['chat_url'],
					'notificationType' => $all_settings['notification_type'],
					'customChatFields' => self::get_form_fields_config( $type ),
					'chatOptions' => array(
						'autostart' => filter_var( $all_settings['auto_start'], FILTER_VALIDATE_BOOLEAN ) ? true : false
					),
					'enableBusinessHours' => filter_var( $all_settings['enable_business_hours_proactive'], FILTER_VALIDATE_BOOLEAN ) ? true : false
				);

				$config['params'] = array(
					'pageId'   => $all_settings['page_id'],
					'profiles' => self::parse_profiles($all_settings['profiles']),
				);
				break;
		}

		if ( 'json' === $format ) {
			$config = json_encode( $config );
		}

		return $config;
	}

	private static function get_form_fields_config( $type ) {
		// Loop through the fields to add.
		$fields       = Forms::get_form_field_settings();
		$field_types  = Forms::form_field_types();
		$found_fields = array();

		foreach( $fields as $key => $field ) {
			if ( ! in_array( $type, $field_types[ $field['field_type'] ]['supports'] ) ) {
				continue;
			}

			switch ( $field['field_type'] ) {

				case 'email':
					if ( is_user_logged_in() ) {
						$current_user = wp_get_current_user();
						$value        = $current_user->user_email;
					} else {
						$value = $field['default_value'];
					}

					if ( 'email' === $type ) {
						$field['show'] = true;
					}
					break;

				case 'name':
					if ( is_user_logged_in() ) {
						$current_user = wp_get_current_user();
						$value        = $current_user->display_name;
					} else {
						$value = $field['default_value'];
					}
					break;

				default:
					$value = $field['default_value'];
			}

			$found_fields[ strtolower( $field['field_name'] ) ] = array(
				'value' => $value,
				'show'  => filter_var( $field['show'], FILTER_VALIDATE_BOOLEAN ) ? true : false,
				'label' => $field['display_label'],
			);

			$_is_required = filter_var( $field['field_required'], FILTER_VALIDATE_BOOLEAN ) ?	true : false;
			if ( $field['field_type'] == 'text' && $_is_required) {
				$found_fields[ strtolower( $field['field_name'] ) ]['required'] = true;
			}
		}

		return $found_fields;
	}

	public static function parse_profiles ( $value ) {
		if (strpos($value, ',') === false) {
			return $value;
		}
	  return array_map('trim', explode(',', $value));
	}

	public function get_widget_types() {
		$widget_types = array(
			'chat'      => array(
				'supported_groups' => array(
					'required',
					'after_chat_survey',
					'chat_after_hours',
					'print',
					'usability',
					'custom_profile_label',
					'style_options',
					'sounds',
					'technical_options',
					'callback_options',
					'auto_start',
				)
			),
			'email'     => array(
				'supported_groups' => array(
					'required', 'style_options',
				)
			),
			'proactive' => array(
				'supported_groups' => array(
					'proactive_chats',
					'auto_start',
				)
			)
		);

		return apply_filters( 'Five9\Settings\get_widget_types', $widget_types );
	}

}

Widget::instance();
